/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2005-2014 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: lm3sxxxx.h 301 2015-01-07 04:57:01Z ertl-ishikawa $
 */

/*
 *  TZ1000に関する定義
 */

#ifndef TOPPERS_TZ1000_H
#define TOPPERS_TZ1000_H

/*
 *  割込み番号の最大値
 */
#define TMAX_INTNO   (15 + 80)

/*
 *  割込み優先度のビット幅
 */
#define TBITW_IPRI     3

/*
 *  割込み優先度ビット幅中のサブ優先度のビット幅
 */
#define TBITW_SUBIPRI  0

#include <sil.h>

#ifndef TOPPERS_MACRO_ONLY

/*
 * INTNO
 */
#define INTNO_UART1     (16 + 41)
#define INTNO_TMR1      (16 + 61)

/*
 * TMR
 */
#define TMR0_BASE           0x40042000
#define TMR1_BASE           0x40042020
#define TMRLOAD(tmr)        ((void *)(tmr + 0x0000U))
#define TMRVALUE(tmr)       ((void *)(tmr + 0x0004U))
#define TMRCONTROL(tmr)     ((void *)(tmr + 0x0008U))
#define TMRINTCLR(tmr)      ((void *)(tmr + 0x000CU))
#define TMRRIS(tmr)         ((void *)(tmr + 0x0010U))
#define TMRMIS(tmr)         ((void *)(tmr + 0x0014U))
#define TMRBGLOAD(tmr)      ((void *)(tmr + 0x0018U))

#define TMRCONTROL_ENABLE   ((uint32_t)(1U << 7))
#define TMRCONTROL_PERIOD   ((uint32_t)(1U << 6))
#define TMRCONTROL_INTENA   ((uint32_t)(1U << 5))
#define TMRCONTROL_DIV_16   ((uint32_t)(1U << 2))
#define TMRCONTROL_DIV_256  ((uint32_t)(2U << 2))
#define TMRCONTROL_32BIT    ((uint32_t)(1U << 1))
#define TMRCONTROL_ONESHOT  ((uint32_t)(1U << 0))

/*
 *  UART
 */
#define UART1_BASE          0x40065000U
#define UARTDR(uart)        ((void *)(uart + 0x0000U))
#define UARTRSR_ECR(uart)   ((void *)(uart + 0x0004U))
#define UARTFR(uart)        ((void *)(uart + 0x0018U))
#define UARTIBRD(uart)      ((void *)(uart + 0x0024U))
#define UARTFBRD(uart)      ((void *)(uart + 0x0028U))
#define UARTLCR_H(uart)     ((void *)(uart + 0x002CU))
#define UARTCR(uart)        ((void *)(uart + 0x0030U))
#define UARTIFLS(uart)      ((void *)(uart + 0x0034U))
#define UARTIMSC(uart)      ((void *)(uart + 0x0038U))
#define UARTRIS(uart)       ((void *)(uart + 0x003CU))
#define UARTMIS(uart)       ((void *)(uart + 0x0040U))
#define UARTICR(uart)       ((void *)(uart + 0x0044U))
#define UARTDMACR(uart)     ((void *)(uart + 0x0048U))

#define UARTFR_TXFE ((uint32_t)(1U << 7))
#define UARTFR_RXFF ((uint32_t)(1U << 6))
#define UARTFR_TXFF ((uint32_t)(1U << 5))
#define UARTFR_RXFE ((uint32_t)(1U << 4))

/*
 *  受信割込みを発生させる閾値の受信FIFO Full byte数
 */
#define UARTIFLS_RX_1       ((uint32_t)(0U << 3))
#define UARTIFLS_RX_2       ((uint32_t)(1U << 3))
#define UARTIFLS_RX_4       ((uint32_t)(2U << 3))
#define UARTIFLS_RX_6       ((uint32_t)(3U << 3))
#define UARTIFLS_RX_7       ((uint32_t)(4U << 3))

/*
 *  送信可能割込みを発生させる閾値の送信FIFO Empty byte数
 */
#define UARTIFLS_TX_14      ((uint32_t)(0U))
#define UARTIFLS_TX_12      ((uint32_t)(1U))
#define UARTIFLS_TX_8       ((uint32_t)(2U))
#define UARTIFLS_TX_4       ((uint32_t)(3U))
#define UARTIFLS_TX_2       ((uint32_t)(4U))

#define UARTIMSC_TXIM       ((uint32_t)(1U << 5))
#define UARTIMSC_RXIM       ((uint32_t)(1U << 4))

#define UARTICR_TXIC        ((uint32_t)(1U << 5))
#define UARTICR_RXIC        ((uint32_t)(1U << 4))

/*
 *  東芝製ドライバを使用するためのI/F
 */
#define _STDINT_H
#include "PMU_TZ10xx.h"
#include "GPIO_TZ10xx.h"
#include "TMR_TZ10xx.h"
#include "Driver_UART.h"
extern TZ10XX_DRIVER_PMU Driver_PMU;
extern TZ10XX_DRIVER_GPIO Driver_GPIO;
extern ARM_DRIVER_UART Driver_UART1;
extern TZ10XX_DRIVER_TMR Driver_TMR0;
extern TZ10XX_DRIVER_TMR Driver_TMR1;

#endif /* TOPPERS_MACRO_ONLY */

#endif /* TOPPERS_TZ1000_H */
